@@ -1,4 +1,5 @@
-use super::Compression;
+use super::{Compression, Predicate};
+use crate::compression::predicate::DefaultPredicate;
 use crate::compression_utils::AcceptEncoding;
 use tower_layer::Layer;
 
@@ -9,18 +10,22 @@ use tower_layer::Layer;
 ///
 /// See the [module docs](crate::compression) for more details.
 #[derive(Clone, Debug, Default)]
-pub struct CompressionLayer {
-    _priv: (),
+pub struct CompressionLayer<P = DefaultPredicate> {
     accept: AcceptEncoding,
+    predicate: P,
 }
 
-impl<S> Layer<S> for CompressionLayer {
-    type Service = Compression<S>;
+impl<S, P> Layer<S> for CompressionLayer<P>
+where
+    P: Predicate,
+{
+    type Service = Compression<S, P>;
 
     fn layer(&self, inner: S) -> Self::Service {
         Compression {
             inner,
             accept: self.accept,
+            predicate: self.predicate.clone(),
         }
     }
 }
@@ -78,6 +83,19 @@ impl CompressionLayer {
         self.accept.set_br(false);
         self
     }
+
+    /// Replace the current compression predicate.
+    ///
+    /// See [`Compression::compress_when`] for more details.
+    pub fn compress_when<C>(self, predicate: C) -> CompressionLayer<C>
+    where
+        C: Predicate,
+    {
+        CompressionLayer {
+            accept: self.accept,
+            predicate,
+        }
+    }
 }
 
 #[cfg(test)]
@@ -105,7 +123,7 @@ mod tests {
     }
 
     #[tokio::test]
-    async fn accept_encoding_configuration_works() -> Result<(), Box<dyn std::error::Error>> {
+    async fn accept_encoding_configuration_works() -> Result<(), crate::BoxError> {
         let deflate_only_layer = CompressionLayer::new().no_br().no_gzip();
 
         let mut service = ServiceBuilder::new()
